@@ -13,6 +13,8 @@ import struct
 import sys
 import warnings
 from copy import deepcopy
+from os.path import splitext
+from pathlib import Path, PurePath
 
 from IPython.utils.py3compat import cast_unicode
 from IPython.testing.skipdoctest import skip_doctest
@@ -238,16 +240,22 @@ def display(*objs, include=None, exclude=None, metadata=None, transient=None, di
     want to use. Here is a list of the names of the special methods and the
     values they must return:
 
-      - `_repr_html_`: return raw HTML as a string
-      - `_repr_json_`: return a JSONable dict
-      - `_repr_jpeg_`: return raw JPEG data
-      - `_repr_png_`: return raw PNG data
-      - `_repr_svg_`: return raw SVG data as a string
-      - `_repr_latex_`: return LaTeX commands in a string surrounded by "$".
+      - `_repr_html_`: return raw HTML as a string, or a tuple (see below).
+      - `_repr_json_`: return a JSONable dict, or a tuple (see below).
+      - `_repr_jpeg_`: return raw JPEG data, or a tuple (see below).
+      - `_repr_png_`: return raw PNG data, or a tuple (see below).
+      - `_repr_svg_`: return raw SVG data as a string, or a tuple (see below).
+      - `_repr_latex_`: return LaTeX commands in a string surrounded by "$",
+                        or a tuple (see below).
       - `_repr_mimebundle_`: return a full mimebundle containing the mapping
                              from all mimetypes to data.
                              Use this for any mime-type not listed above.
 
+    The above functions may also return the object's metadata alonside the
+    data.  If the metadata is available, the functions will return a tuple
+    containing the data and metadata, in that order.  If there is no metadata
+    available, then the functions will return the data only.
+
     When you are directly writing your own classes, you can adapt them for
     display in IPython by following the above approach. But in practice, you
     often need to work with existing classes that you can't easily modify.
@@ -288,6 +296,13 @@ def display(*objs, include=None, exclude=None, metadata=None, transient=None, di
     if transient:
         kwargs['transient'] = transient
 
+    if not objs and display_id:
+        # if given no objects, but still a request for a display_id,
+        # we assume the user wants to insert an empty output that
+        # can be updated later
+        objs = [{}]
+        raw = True
+
     if not raw:
         format = InteractiveShell.instance().display_formatter.format
 
@@ -587,6 +602,9 @@ class DisplayObject(object):
         metadata : dict
             Dict of metadata associated to be the object when displayed
         """
+        if isinstance(data, (Path, PurePath)):
+            data = str(data)
+
         if data is not None and isinstance(data, str):
             if data.startswith('http') and url is None:
                 url = data
@@ -666,6 +684,23 @@ class Pretty(TextDisplayObject):
 
 class HTML(TextDisplayObject):
 
+    def __init__(self, data=None, url=None, filename=None, metadata=None):
+        def warn():
+            if not data:
+                return False
+
+            #
+            # Avoid calling lower() on the entire data, because it could be a
+            # long string and we're only interested in its beginning and end.
+            #
+            prefix = data[:10].lower()
+            suffix = data[-10:].lower()
+            return prefix.startswith("<iframe ") and suffix.endswith("</iframe>")
+
+        if warn():
+            warnings.warn("Consider using IPython.display.IFrame instead")
+        super(HTML, self).__init__(data=data, url=url, filename=filename, metadata=metadata)
+
     def _repr_html_(self):
         return self._data_and_metadata()
 
@@ -687,7 +722,7 @@ class Markdown(TextDisplayObject):
 class Math(TextDisplayObject):
 
     def _repr_latex_(self):
-        s = "$$%s$$" % self.data.strip('$')
+        s = r"$\displaystyle %s$" % self.data.strip('$')
         if self.metadata:
             return s, deepcopy(self.metadata)
         else:
@@ -840,6 +875,9 @@ class JSON(DisplayObject):
 
     @data.setter
     def data(self, data):
+        if isinstance(data, (Path, PurePath)):
+            data = str(data)
+
         if isinstance(data, str):
             if getattr(self, 'filename', None) is None:
                 warnings.warn("JSON expects JSONable dict or list, not JSON strings")
@@ -1110,6 +1148,9 @@ class Image(DisplayObject):
         Image(url='http://www.google.fr/images/srpr/logo3w.png')
 
         """
+        if isinstance(data, (Path, PurePath)):
+            data = str(data)
+
         if filename is not None:
             ext = self._find_ext(filename)
         elif url is not None:
@@ -1227,7 +1268,11 @@ class Image(DisplayObject):
 
     def _data_and_metadata(self, always_both=False):
         """shortcut for returning metadata with shape information, if defined"""
-        b64_data = b2a_base64(self.data).decode('ascii')
+        try:
+            b64_data = b2a_base64(self.data).decode('ascii')
+        except TypeError:
+            raise FileNotFoundError(
+                "No such file or directory: '%s'" % (self.data))
         md = {}
         if self.metadata:
             md.update(self.metadata)
@@ -1251,12 +1296,19 @@ class Image(DisplayObject):
             return self._data_and_metadata()
 
     def _find_ext(self, s):
-        return s.split('.')[-1].lower()
+        base, ext = splitext(s)
+
+        if not ext:
+            return base
+
+        # `splitext` includes leading period, so we skip it
+        return ext[1:].lower()
 
 
 class Video(DisplayObject):
 
-    def __init__(self, data=None, url=None, filename=None, embed=False, mimetype=None):
+    def __init__(self, data=None, url=None, filename=None, embed=False,
+                 mimetype=None, width=None, height=None):
         """Create a video object given raw data or an URL.
 
         When this object is returned by an input cell or passed to the
@@ -1288,6 +1340,12 @@ class Video(DisplayObject):
         mimetype: unicode
             Specify the mimetype for embedded videos.
             Default will be guessed from file extension, if available.
+        width : int
+            Width in pixels to which to constrain the video in HTML.
+            If not supplied, defaults to the width of the video.
+        height : int
+            Height in pixels to which to constrain the video in html.
+            If not supplied, defaults to the height of the video.
 
         Examples
         --------
@@ -1297,6 +1355,9 @@ class Video(DisplayObject):
         Video('path/to/video.mp4', embed=True)
         Video(b'raw-videodata', embed=True)
         """
+        if isinstance(data, (Path, PurePath)):
+            data = str(data)
+
         if url is None and isinstance(data, str) and data.startswith(('http:', 'https:')):
             url = data
             data = None
@@ -1314,16 +1375,24 @@ class Video(DisplayObject):
 
         self.mimetype = mimetype
         self.embed = embed
+        self.width = width
+        self.height = height
         super(Video, self).__init__(data=data, url=url, filename=filename)
 
     def _repr_html_(self):
+        width = height = ''
+        if self.width:
+            width = ' width="%d"' % self.width
+        if self.height:
+            height = ' height="%d"' % self.height
+
         # External URLs and potentially local files are not embedded into the
         # notebook output.
         if not self.embed:
             url = self.url if self.url is not None else self.filename
-            output = """<video src="{0}" controls>
+            output = """<video src="{0}" controls {1} {2}>
       Your browser does not support the <code>video</code> element.
-    </video>""".format(url)
+    </video>""".format(url, width, height)
             return output
 
         # Embedded videos are base64-encoded.
@@ -1342,10 +1411,10 @@ class Video(DisplayObject):
         else:
             b64_video = b2a_base64(video).decode('ascii').rstrip()
 
-        output = """<video controls>
- <source src="data:{0};base64,{1}" type="{0}">
+        output = """<video controls {0} {1}>
+ <source src="data:{2};base64,{3}" type="{2}">
  Your browser does not support the video tag.
- </video>""".format(mimetype, b64_video)
+ </video>""".format(width, height, mimetype, b64_video)
         return output
 
     def reload(self):
