@@ -15,22 +15,20 @@ import json
 import sys
 import traceback
 import warnings
+from io import StringIO
 
 from decorator import decorator
 
 from traitlets.config.configurable import Configurable
-from IPython.core.getipython import get_ipython
-from IPython.utils.sentinel import Sentinel
-from IPython.utils.dir2 import get_real_method
-from IPython.lib import pretty
+from .getipython import get_ipython
+from ..utils.sentinel import Sentinel
+from ..utils.dir2 import get_real_method
+from ..lib import pretty
 from traitlets import (
     Bool, Dict, Integer, Unicode, CUnicode, ObjectName, List,
     ForwardDeclaredInstance,
     default, observe,
 )
-from IPython.utils.py3compat import (
-    with_metaclass, string_types, unicode_type,
-)
 
 
 class DisplayFormatter(Configurable):
@@ -161,8 +159,17 @@ class DisplayFormatter(Configurable):
 
         for format_type, formatter in self.formatters.items():
             if format_type in format_dict:
-                # already got it from mimebundle, don't render again
-                continue
+                # already got it from mimebundle, maybe don't render again.
+                # exception: manually registered per-mime renderer
+                # check priority:
+                # 1. user-registered per-mime formatter
+                # 2. mime-bundle (user-registered or repr method)
+                # 3. default per-mime formatter (e.g. repr method)
+                try:
+                    formatter.lookup(obj)
+                except KeyError:
+                    # no special formatter, use mime-bundle-provided value
+                    continue
             if include and format_type not in include:
                 continue
             if exclude and format_type in exclude:
@@ -217,7 +224,7 @@ def catch_format_error(method, self, *args, **kwargs):
         r = method(self, *args, **kwargs)
     except NotImplementedError:
         # don't warn on NotImplementedErrors
-        return None
+        return self._check_return(None, args[0])
     except Exception:
         exc_info = sys.exc_info()
         ip = get_ipython()
@@ -225,11 +232,11 @@ def catch_format_error(method, self, *args, **kwargs):
             ip.showtraceback(exc_info)
         else:
             traceback.print_exception(*exc_info)
-        return None
+        return self._check_return(None, args[0])
     return self._check_return(r, args[0])
 
 
-class FormatterABC(with_metaclass(abc.ABCMeta, object)):
+class FormatterABC(metaclass=abc.ABCMeta):
     """ Abstract base class for Formatters.
 
     A formatter is a callable class that is responsible for computing the
@@ -303,7 +310,7 @@ class BaseFormatter(Configurable):
     """
 
     format_type = Unicode('text/plain')
-    _return_type = string_types
+    _return_type = str
 
     enabled = Bool(True).tag(config=True)
 
@@ -403,7 +410,7 @@ class BaseFormatter(Configurable):
         ------
         KeyError if the type has not been registered.
         """
-        if isinstance(typ, string_types):
+        if isinstance(typ, str):
             typ_key = tuple(typ.rsplit('.',1))
             if typ_key not in self.deferred_printers:
                 # We may have it cached in the type map. We will have to
@@ -446,7 +453,7 @@ class BaseFormatter(Configurable):
             this will be the previous value (to enable restoring later).
         """
         # if string given, interpret as 'pkg.module.class_name'
-        if isinstance(typ, string_types):
+        if isinstance(typ, str):
             type_module, type_name = typ.rsplit('.', 1)
             return self.for_type_by_name(type_module, type_name, func)
         
@@ -518,7 +525,7 @@ class BaseFormatter(Configurable):
         KeyError if the type is not registered and default is not specified.
         """
         
-        if isinstance(typ, string_types):
+        if isinstance(typ, str):
             typ_key = tuple(typ.rsplit('.',1))
             if typ_key not in self.deferred_printers:
                 # We may have it cached in the type map. We will have to
@@ -685,11 +692,7 @@ class PlainTextFormatter(BaseFormatter):
         if not self.pprint:
             return repr(obj)
         else:
-            # handle str and unicode on Python 2
-            # io.StringIO only accepts unicode,
-            # cStringIO doesn't handle unicode on py2,
-            # StringIO allows str, unicode but only ascii str
-            stream = pretty.CUnicodeIO()
+            stream = StringIO()
             printer = pretty.RepresentationPrinter(stream, self.verbose,
                 self.max_width, self.newline,
                 max_seq_length=self.max_seq_length,
@@ -764,7 +767,7 @@ class PNGFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_png_')
     
-    _return_type = (bytes, unicode_type)
+    _return_type = (bytes, str)
 
 
 class JPEGFormatter(BaseFormatter):
@@ -782,7 +785,7 @@ class JPEGFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_jpeg_')
 
-    _return_type = (bytes, unicode_type)
+    _return_type = (bytes, str)
 
 
 class LatexFormatter(BaseFormatter):
@@ -831,7 +834,7 @@ class JSONFormatter(BaseFormatter):
             r, md = r
         
         # handle deprecated JSON-as-string form from IPython < 3
-        if isinstance(r, string_types):
+        if isinstance(r, str):
             warnings.warn("JSON expects JSONable list/dict containers, not JSON strings",
             FormatterWarning)
             r = json.loads(r)
@@ -873,7 +876,7 @@ class PDFFormatter(BaseFormatter):
 
     print_method = ObjectName('_repr_pdf_')
 
-    _return_type = (bytes, unicode_type)
+    _return_type = (bytes, str)
 
 class IPythonDisplayFormatter(BaseFormatter):
     """An escape-hatch Formatter for objects that know how to display themselves.
@@ -964,10 +967,7 @@ class MimeBundleFormatter(BaseFormatter):
             method = get_real_method(obj, self.print_method)
 
             if method is not None:
-                d = {}
-                d['include'] = include
-                d['exclude'] = exclude
-                return method(**d)
+                return method(include=include, exclude=exclude)
             return None
         else:
             return None
@@ -993,19 +993,6 @@ def format_display_data(obj, include=None, exclude=None):
 
     By default all format types will be computed.
 
-    The following MIME types are currently implemented:
-
-    * text/plain
-    * text/html
-    * text/markdown
-    * text/latex
-    * application/json
-    * application/javascript
-    * application/pdf
-    * image/png
-    * image/jpeg
-    * image/svg+xml
-
     Parameters
     ----------
     obj : object
@@ -1024,11 +1011,11 @@ def format_display_data(obj, include=None, exclude=None):
         format data dict. If this is set *only* the format types included
         in this list will be computed.
     exclude : list or tuple, optional
-        A list of format type string (MIME types) to exclue in the format
+        A list of format type string (MIME types) to exclude in the format
         data dict. If this is set all format types will be computed,
         except for those included in this argument.
     """
-    from IPython.core.interactiveshell import InteractiveShell
+    from .interactiveshell import InteractiveShell
 
     return InteractiveShell.instance().display_formatter.format(
         obj,
