@@ -7,30 +7,28 @@ launch InteractiveShell instances, load extensions, etc.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from __future__ import absolute_import
-from __future__ import print_function
-
 import glob
+from itertools import chain
 import os
 import sys
 
 from traitlets.config.application import boolean_flag
 from traitlets.config.configurable import Configurable
 from traitlets.config.loader import Config
+from IPython.core.application import SYSTEM_CONFIG_DIRS, ENV_CONFIG_DIRS
 from IPython.core import pylabtools
-from IPython.utils import py3compat
 from IPython.utils.contexts import preserve_keys
 from IPython.utils.path import filefind
 from traitlets import (
-    Unicode, Instance, List, Bool, CaselessStrEnum
+    Unicode, Instance, List, Bool, CaselessStrEnum, observe,
 )
-from IPython.lib.inputhook import guis
+from IPython.terminal import pt_inputhooks
 
 #-----------------------------------------------------------------------------
 # Aliases and Flags
 #-----------------------------------------------------------------------------
 
-gui_keys = tuple(sorted([ key for key in guis if key is not None ]))
+gui_keys = tuple(sorted(pt_inputhooks.backends) + sorted(pt_inputhooks.aliases))
 
 backend_keys = sorted(pylabtools.backends.keys())
 backend_keys.insert(0, 'auto')
@@ -50,14 +48,6 @@ addflag('pdb', 'InteractiveShell.pdb',
     "Enable auto calling the pdb debugger after every exception.",
     "Disable auto calling the pdb debugger after every exception."
 )
-# pydb flag doesn't do any config, as core.debugger switches on import,
-# which is before parsing.  This just allows the flag to be passed.
-shell_flags.update(dict(
-    pydb = ({},
-        """Use the third party 'pydb' package as debugger, instead of pdb.
-        Requires that pydb is installed."""
-    )
-))
 addflag('pprint', 'PlainTextFormatter.pprint',
     "Enable auto pretty printing of results.",
     "Disable auto pretty printing of results."
@@ -70,17 +60,9 @@ addflag('color-info', 'InteractiveShell.color_info',
     colours.""",
     "Disable using colors for info related things."
 )
-addflag('deep-reload', 'InteractiveShell.deep_reload',
-    """ **Deprecated** Enable deep (recursive) reloading by default. IPython can use the
-    deep_reload module which reloads changes in modules recursively (it
-    replaces the reload() function, so you don't need to change anything to
-    use it). deep_reload() forces a full reload of modules whose code may
-    have changed, which the default reload() function does not.  When
-    deep_reload is off, IPython will use the normal reload(), but
-    deep_reload will still be available as dreload(). This feature is off
-    by default [which means that you have both normal reload() and
-    dreload()].""",
-    "Disable deep (recursive) reloading by default."
+addflag('ignore-cwd', 'InteractiveShellApp.ignore_cwd',
+        "Exclude the current working directory from sys.path",
+        "Include the current working directory in sys.path",
 )
 nosep_config = Config()
 nosep_config.InteractiveShell.separate_in = ''
@@ -120,7 +102,7 @@ shell_aliases['cache-size'] = 'InteractiveShell.cache_size'
 
 class InteractiveShellApp(Configurable):
     """A Mixin for applications that start InteractiveShell instances.
-    
+
     Provides configurables for loading extensions and executing files
     as part of configuring a Shell environment.
 
@@ -133,80 +115,104 @@ class InteractiveShellApp(Configurable):
       - :meth:`init_extensions`
       - :meth:`init_code`
     """
-    extensions = List(Unicode(), config=True,
+    extensions = List(Unicode(),
         help="A list of dotted module names of IPython extensions to load."
-    )
-    extra_extension = Unicode('', config=True,
+    ).tag(config=True)
+    extra_extension = Unicode('',
         help="dotted module name of an IPython extension to load."
-    )
+    ).tag(config=True)
 
-    reraise_ipython_extension_failures = Bool(
-        False,
-        config=True,
+    reraise_ipython_extension_failures = Bool(False,
         help="Reraise exceptions encountered loading IPython extensions?",
-    )
+    ).tag(config=True)
 
     # Extensions that are always loaded (not configurable)
-    default_extensions = List(Unicode(), [u'storemagic'], config=False)
-    
-    hide_initial_ns = Bool(True, config=True,
+    default_extensions = List(Unicode(), [u'storemagic']).tag(config=False)
+
+    hide_initial_ns = Bool(True,
         help="""Should variables loaded at startup (by startup files, exec_lines, etc.)
         be hidden from tools like %who?"""
-    )
+    ).tag(config=True)
 
-    exec_files = List(Unicode(), config=True,
+    exec_files = List(Unicode(),
         help="""List of files to run at IPython startup."""
-    )
-    exec_PYTHONSTARTUP = Bool(True, config=True,
+    ).tag(config=True)
+    exec_PYTHONSTARTUP = Bool(True,
         help="""Run the file referenced by the PYTHONSTARTUP environment
         variable at IPython startup."""
-    )
-    file_to_run = Unicode('', config=True,
-        help="""A file to be run""")
+    ).tag(config=True)
+    file_to_run = Unicode('',
+        help="""A file to be run""").tag(config=True)
 
-    exec_lines = List(Unicode(), config=True,
+    exec_lines = List(Unicode(),
         help="""lines of code to run at IPython startup."""
-    )
-    code_to_run = Unicode('', config=True,
+    ).tag(config=True)
+    code_to_run = Unicode('',
         help="Execute the given command string."
-    )
-    module_to_run = Unicode('', config=True,
+    ).tag(config=True)
+    module_to_run = Unicode('',
         help="Run the module as a script."
-    )
-    gui = CaselessStrEnum(gui_keys, config=True, allow_none=True,
+    ).tag(config=True)
+    gui = CaselessStrEnum(gui_keys, allow_none=True,
         help="Enable GUI event loop integration with any of {0}.".format(gui_keys)
-    )
+    ).tag(config=True)
     matplotlib = CaselessStrEnum(backend_keys, allow_none=True,
-        config=True,
         help="""Configure matplotlib for interactive use with
         the default matplotlib backend."""
-    )
+    ).tag(config=True)
     pylab = CaselessStrEnum(backend_keys, allow_none=True,
-        config=True,
         help="""Pre-load matplotlib and numpy for interactive use,
         selecting a particular matplotlib backend and loop integration.
         """
-    )
-    pylab_import_all = Bool(True, config=True,
+    ).tag(config=True)
+    pylab_import_all = Bool(True,
         help="""If true, IPython will populate the user namespace with numpy, pylab, etc.
         and an ``import *`` is done from numpy and pylab, when using pylab mode.
-        
+
         When False, pylab mode should not import any names into the user namespace.
         """
-    )
+    ).tag(config=True)
+    ignore_cwd = Bool(
+        False,
+        help="""If True, IPython will not add the current working directory to sys.path.
+        When False, the current working directory is added to sys.path, allowing imports
+        of modules defined in the current directory."""
+    ).tag(config=True)
     shell = Instance('IPython.core.interactiveshell.InteractiveShellABC',
                      allow_none=True)
-    
+    # whether interact-loop should start
+    interact = Bool(True)
+
     user_ns = Instance(dict, args=None, allow_none=True)
-    def _user_ns_changed(self, name, old, new):
+    @observe('user_ns')
+    def _user_ns_changed(self, change):
         if self.shell is not None:
-            self.shell.user_ns = new
+            self.shell.user_ns = change['new']
             self.shell.init_user_ns()
 
     def init_path(self):
-        """Add current working directory, '', to sys.path"""
-        if sys.path[0] != '':
-            sys.path.insert(0, '')
+        """Add current working directory, '', to sys.path
+
+        Unlike Python's default, we insert before the first `site-packages`
+        or `dist-packages` directory,
+        so that it is after the standard library.
+
+        .. versionchanged:: 7.2
+            Try to insert after the standard library, instead of first.
+        .. versionchanged:: 8.0
+            Allow optionally not including the current directory in sys.path
+        """
+        if '' in sys.path or self.ignore_cwd:
+            return
+        for idx, path in enumerate(sys.path):
+            parent, last_part = os.path.split(path)
+            if last_part in {'site-packages', 'dist-packages'}:
+                break
+        else:
+            # no site-packages or dist-packages found (?!)
+            # back to original behavior of inserting at the front
+            idx = 0
+        sys.path.insert(idx, '')
 
     def init_shell(self):
         raise NotImplementedError("Override in subclasses")
@@ -224,21 +230,21 @@ class InteractiveShellApp(Configurable):
         elif self.gui:
             enable = shell.enable_gui
             key = self.gui
-        
+
         if not enable:
             return
-        
+
         try:
             r = enable(key)
         except ImportError:
-            self.log.warn("Eventloop or matplotlib integration failed. Is matplotlib installed?")
+            self.log.warning("Eventloop or matplotlib integration failed. Is matplotlib installed?")
             self.shell.showtraceback()
             return
         except Exception:
-            self.log.warn("GUI event loop or pylab initialization failed")
+            self.log.warning("GUI event loop or pylab initialization failed")
             self.shell.showtraceback()
             return
-            
+
         if isinstance(r, tuple):
             gui, backend = r[:2]
             self.log.info("Enabling GUI event loop integration, "
@@ -273,27 +279,27 @@ class InteractiveShellApp(Configurable):
                                ext=ext,
                                location=self.profile_dir.location
                            ))
-                    self.log.warn(msg, exc_info=True)
+                    self.log.warning(msg, exc_info=True)
         except:
             if self.reraise_ipython_extension_failures:
                 raise
-            self.log.warn("Unknown error in loading extensions:", exc_info=True)
+            self.log.warning("Unknown error in loading extensions:", exc_info=True)
 
     def init_code(self):
         """run the pre-flight code, specified via exec_lines"""
         self._run_startup_files()
         self._run_exec_lines()
         self._run_exec_files()
-        
+
         # Hide variables defined here from %who etc.
         if self.hide_initial_ns:
             self.shell.user_ns_hidden.update(self.shell.user_ns)
-        
+
         # command-line execution (ipython -i script.py, ipython -m module)
         # should *not* be excluded from %whos
         self._run_cmd_line_code()
         self._run_module()
-        
+
         # flush output, so itwon't be attached to the first cell
         sys.stdout.flush()
         sys.stderr.flush()
@@ -310,26 +316,23 @@ class InteractiveShellApp(Configurable):
                                   line)
                     self.shell.run_cell(line, store_history=False)
                 except:
-                    self.log.warn("Error in executing line in user "
+                    self.log.warning("Error in executing line in user "
                                   "namespace: %s" % line)
                     self.shell.showtraceback()
         except:
-            self.log.warn("Unknown error in handling IPythonApp.exec_lines:")
+            self.log.warning("Unknown error in handling IPythonApp.exec_lines:")
             self.shell.showtraceback()
 
     def _exec_file(self, fname, shell_futures=False):
         try:
             full_filename = filefind(fname, [u'.', self.ipython_dir])
-        except IOError as e:
-            self.log.warn("File not found: %r"%fname)
+        except IOError:
+            self.log.warning("File not found: %r"%fname)
             return
         # Make sure that the running script gets a proper sys.argv as if it
         # were run from a system shell.
         save_argv = sys.argv
         sys.argv = [full_filename] + self.extra_args[1:]
-        # protect sys.argv from potential unicode strings on Python 2:
-        if not py3compat.PY3:
-            sys.argv = [ py3compat.cast_bytes(a) for a in sys.argv ]
         try:
             if os.path.isfile(full_filename):
                 self.log.info("Running file in user namespace: %s" %
@@ -338,22 +341,25 @@ class InteractiveShellApp(Configurable):
                 # behavior.
                 with preserve_keys(self.shell.user_ns, '__file__'):
                     self.shell.user_ns['__file__'] = fname
-                    if full_filename.endswith('.ipy'):
+                    if full_filename.endswith('.ipy') or full_filename.endswith('.ipynb'):
                         self.shell.safe_execfile_ipy(full_filename,
                                                      shell_futures=shell_futures)
                     else:
                         # default to python, even without extension
                         self.shell.safe_execfile(full_filename,
                                                  self.shell.user_ns,
-                                                 shell_futures=shell_futures)
+                                                 shell_futures=shell_futures,
+                                                 raise_exceptions=True)
         finally:
             sys.argv = save_argv
 
     def _run_startup_files(self):
         """Run files from profile startup directory"""
-        startup_dir = self.profile_dir.startup_dir
+        startup_dirs = [self.profile_dir.startup_dir] + [
+            os.path.join(p, 'startup') for p in chain(ENV_CONFIG_DIRS, SYSTEM_CONFIG_DIRS)
+        ]
         startup_files = []
-        
+
         if self.exec_PYTHONSTARTUP and os.environ.get('PYTHONSTARTUP', False) and \
                 not (self.file_to_run or self.code_to_run or self.module_to_run):
             python_startup = os.environ['PYTHONSTARTUP']
@@ -361,26 +367,20 @@ class InteractiveShellApp(Configurable):
             try:
                 self._exec_file(python_startup)
             except:
-                self.log.warn("Unknown error in handling PYTHONSTARTUP file %s:", python_startup)
+                self.log.warning("Unknown error in handling PYTHONSTARTUP file %s:", python_startup)
                 self.shell.showtraceback()
-            finally:
-                # Many PYTHONSTARTUP files set up the readline completions,
-                # but this is often at odds with IPython's own completions.
-                # Do not allow PYTHONSTARTUP to set up readline.
-                if self.shell.has_readline:
-                    self.shell.set_readline_completer()
-        
-        startup_files += glob.glob(os.path.join(startup_dir, '*.py'))
-        startup_files += glob.glob(os.path.join(startup_dir, '*.ipy'))
+        for startup_dir in startup_dirs[::-1]:
+            startup_files += glob.glob(os.path.join(startup_dir, '*.py'))
+            startup_files += glob.glob(os.path.join(startup_dir, '*.ipy'))
         if not startup_files:
             return
-        
+
         self.log.debug("Running startup files from %s...", startup_dir)
         try:
             for fname in sorted(startup_files):
                 self._exec_file(fname)
         except:
-            self.log.warn("Unknown error in handling startup files:")
+            self.log.warning("Unknown error in handling startup files:")
             self.shell.showtraceback()
 
     def _run_exec_files(self):
@@ -393,7 +393,7 @@ class InteractiveShellApp(Configurable):
             for fname in self.exec_files:
                 self._exec_file(fname)
         except:
-            self.log.warn("Unknown error in handling IPythonApp.exec_files:")
+            self.log.warning("Unknown error in handling IPythonApp.exec_files:")
             self.shell.showtraceback()
 
     def _run_cmd_line_code(self):
@@ -405,19 +405,26 @@ class InteractiveShellApp(Configurable):
                               line)
                 self.shell.run_cell(line, store_history=False)
             except:
-                self.log.warn("Error in executing line in user namespace: %s" %
+                self.log.warning("Error in executing line in user namespace: %s" %
                               line)
                 self.shell.showtraceback()
+                if not self.interact:
+                    self.exit(1)
 
         # Like Python itself, ignore the second if the first of these is present
         elif self.file_to_run:
             fname = self.file_to_run
+            if os.path.isdir(fname):
+                fname = os.path.join(fname, "__main__.py")
+            if not os.path.exists(fname):
+                self.log.warning("File '%s' doesn't exist", fname)
+                self.exit(2)
             try:
                 self._exec_file(fname, shell_futures=True)
             except:
-                self.log.warn("Error in executing file in user namespace: %s" %
-                              fname)
-                self.shell.showtraceback()
+                self.shell.showtraceback(tb_offset=4)
+                if not self.interact:
+                    self.exit(1)
 
     def _run_module(self):
         """Run module specified at the command-line."""
