@@ -9,7 +9,9 @@ import os
 import re
 import sys
 import warnings
+from textwrap import dedent
 from unittest import TestCase
+from unittest import mock
 from importlib import invalidate_caches
 from io import StringIO
 
@@ -23,17 +25,15 @@ from IPython.core.error import UsageError
 from IPython.core.magic import (Magics, magics_class, line_magic,
                                 cell_magic,
                                 register_line_magic, register_cell_magic)
-from IPython.core.magics import execution, script, code, logging
+from IPython.core.magics import execution, script, code, logging, osm
 from IPython.testing import decorators as dec
 from IPython.testing import tools as tt
 from IPython.utils.io import capture_output
-from IPython.utils.tempdir import TemporaryDirectory
+from IPython.utils.tempdir import (TemporaryDirectory,
+                                    TemporaryWorkingDirectory)
 from IPython.utils.process import find_cmd
 
 
-
-_ip = get_ipython()
-
 @magic.magics_class
 class DummyMagics(magic.Magics): pass
 
@@ -148,6 +148,7 @@ def test_rehashx():
     # rehashx must fill up syscmdlist
     scoms = _ip.db['syscmdlist']
     nt.assert_true(len(scoms) > 10)
+            
 
 
 def test_magic_parse_options():
@@ -369,6 +370,31 @@ def test_reset_in_length():
     _ip.run_cell("reset -f in")
     nt.assert_equal(len(_ip.user_ns['In']), _ip.displayhook.prompt_count+1)
 
+class TestResetErrors(TestCase):
+
+    def test_reset_redefine(self):
+
+        @magics_class
+        class KernelMagics(Magics):
+              @line_magic
+              def less(self, shell): pass
+
+        _ip.register_magics(KernelMagics)
+
+        with self.assertLogs() as cm:
+            # hack, we want to just capture logs, but assertLogs fails if not
+            # logs get produce.
+            # so log one things we ignore.
+            import logging as log_mod
+            log = log_mod.getLogger()
+            log.info('Nothing')
+            # end hack.
+            _ip.run_cell("reset -f")
+
+        assert len(cm.output) == 1
+        for out in cm.output:
+            assert "Invalid alias" not in out
+
 def test_tb_syntaxerror():
     """test %tb after a SyntaxError"""
     ip = get_ipython()
@@ -400,6 +426,15 @@ def test_time():
         with tt.AssertPrints("hihi", suppress=False):
             ip.run_cell("f('hi')")
 
+def test_time_last_not_expression():
+    ip.run_cell("%%time\n"
+                "var_1 = 1\n"
+                "var_2 = 2\n")
+    assert ip.user_ns['var_1'] == 1
+    del ip.user_ns['var_1']
+    assert ip.user_ns['var_2'] == 2
+    del ip.user_ns['var_2']
+    
 
 @dec.skip_win32
 def test_time2():
@@ -418,6 +453,29 @@ def test_time3():
                     "run = 0\n"
                     "run += 1")
 
+def test_multiline_time():
+    """Make sure last statement from time return a value."""
+    ip = get_ipython()
+    ip.user_ns.pop('run', None)
+
+    ip.run_cell(dedent("""\
+        %%time
+        a = "ho"
+        b = "hey"
+        a+b
+        """))
+    nt.assert_equal(ip.user_ns_hidden['_'], 'hohey')
+
+def test_time_local_ns():
+    """
+    Test that local_ns is actually global_ns when running a cell magic
+    """
+    ip = get_ipython()
+    ip.run_cell("%%time\n"
+                "myvar = 1")
+    nt.assert_equal(ip.user_ns['myvar'], 1)
+    del ip.user_ns['myvar']
+
 def test_doctest_mode():
     "Toggle doctest_mode twice, it should be a no-op and run without error"
     _ip.magic('doctest_mode')
@@ -432,7 +490,7 @@ def test_parse_options():
     nt.assert_equal(m.parse_options('foo', '')[1], 'foo')
     nt.assert_equal(m.parse_options(u'foo', '')[1], u'foo')
 
-    
+
 def test_dirops():
     """Test various directory handling operations."""
     # curpath = lambda :os.path.splitdrive(os.getcwd())[1].replace('\\','/')
@@ -452,10 +510,27 @@ def test_dirops():
         os.chdir(startdir)
 
 
+def test_cd_force_quiet():
+    """Test OSMagics.cd_force_quiet option"""
+    _ip.config.OSMagics.cd_force_quiet = True
+    osmagics = osm.OSMagics(shell=_ip)
+
+    startdir = os.getcwd()
+    ipdir = os.path.realpath(_ip.ipython_dir)
+
+    try:
+        with tt.AssertNotPrints(ipdir):
+            osmagics.cd('"%s"' % ipdir)
+        with tt.AssertNotPrints(startdir):
+            osmagics.cd('-')
+    finally:
+        os.chdir(startdir)
+
+
 def test_xmode():
     # Calling xmode three times should be a no-op
     xmode = _ip.InteractiveTB.mode
-    for i in range(3):
+    for i in range(4):
         _ip.magic("xmode")
     nt.assert_equal(_ip.InteractiveTB.mode, xmode)
     
@@ -659,6 +734,24 @@ class TestEnv(TestCase):
         env = _ip.magic("env")
         self.assertTrue(isinstance(env, dict))
 
+    def test_env_secret(self):
+        env = _ip.magic("env")
+        hidden = "<hidden>"
+        with mock.patch.dict(
+            os.environ,
+            {
+                "API_KEY": "abc123",
+                "SECRET_THING": "ssshhh",
+                "JUPYTER_TOKEN": "",
+                "VAR": "abc"
+            }
+        ):
+            env = _ip.magic("env")
+        assert env["API_KEY"] == hidden
+        assert env["SECRET_THING"] == hidden
+        assert env["JUPYTER_TOKEN"] == hidden
+        assert env["VAR"] == "abc"
+
     def test_env_get_set_simple(self):
         env = _ip.magic("env var val1")
         self.assertEqual(env, None)
@@ -738,11 +831,41 @@ class CellMagicTestCase(TestCase):
         nt.assert_equal(c33, None)
 
 def test_file():
-    """Basic %%file"""
+    """Basic %%writefile"""
     ip = get_ipython()
     with TemporaryDirectory() as td:
         fname = os.path.join(td, 'file1')
-        ip.run_cell_magic("file", fname, u'\n'.join([
+        ip.run_cell_magic("writefile", fname, u'\n'.join([
+            'line1',
+            'line2',
+        ]))
+        with open(fname) as f:
+            s = f.read()
+        nt.assert_in('line1\n', s)
+        nt.assert_in('line2', s)
+
+@dec.skip_win32
+def test_file_single_quote():
+    """Basic %%writefile with embedded single quotes"""
+    ip = get_ipython()
+    with TemporaryDirectory() as td:
+        fname = os.path.join(td, '\'file1\'')
+        ip.run_cell_magic("writefile", fname, u'\n'.join([
+            'line1',
+            'line2',
+        ]))
+        with open(fname) as f:
+            s = f.read()
+        nt.assert_in('line1\n', s)
+        nt.assert_in('line2', s)
+
+@dec.skip_win32
+def test_file_double_quote():
+    """Basic %%writefile with embedded double quotes"""
+    ip = get_ipython()
+    with TemporaryDirectory() as td:
+        fname = os.path.join(td, '"file1"')
+        ip.run_cell_magic("writefile", fname, u'\n'.join([
             'line1',
             'line2',
         ]))
@@ -752,12 +875,12 @@ def test_file():
         nt.assert_in('line2', s)
 
 def test_file_var_expand():
-    """%%file $filename"""
+    """%%writefile $filename"""
     ip = get_ipython()
     with TemporaryDirectory() as td:
         fname = os.path.join(td, 'file1')
         ip.user_ns['filename'] = fname
-        ip.run_cell_magic("file", '$filename', u'\n'.join([
+        ip.run_cell_magic("writefile", '$filename', u'\n'.join([
             'line1',
             'line2',
         ]))
@@ -767,11 +890,11 @@ def test_file_var_expand():
         nt.assert_in('line2', s)
 
 def test_file_unicode():
-    """%%file with unicode cell"""
+    """%%writefile with unicode cell"""
     ip = get_ipython()
     with TemporaryDirectory() as td:
         fname = os.path.join(td, 'file1')
-        ip.run_cell_magic("file", fname, u'\n'.join([
+        ip.run_cell_magic("writefile", fname, u'\n'.join([
             u'liné1',
             u'liné2',
         ]))
@@ -781,15 +904,15 @@ def test_file_unicode():
         nt.assert_in(u'liné2', s)
 
 def test_file_amend():
-    """%%file -a amends files"""
+    """%%writefile -a amends files"""
     ip = get_ipython()
     with TemporaryDirectory() as td:
         fname = os.path.join(td, 'file2')
-        ip.run_cell_magic("file", fname, u'\n'.join([
+        ip.run_cell_magic("writefile", fname, u'\n'.join([
             'line1',
             'line2',
         ]))
-        ip.run_cell_magic("file", "-a %s" % fname, u'\n'.join([
+        ip.run_cell_magic("writefile", "-a %s" % fname, u'\n'.join([
             'line3',
             'line4',
         ]))
@@ -797,7 +920,20 @@ def test_file_amend():
             s = f.read()
         nt.assert_in('line1\n', s)
         nt.assert_in('line3\n', s)
-        
+
+def test_file_spaces():
+    """%%file with spaces in filename"""
+    ip = get_ipython()
+    with TemporaryWorkingDirectory() as td:
+        fname = "file name"
+        ip.run_cell_magic("file", '"%s"'%fname, u'\n'.join([
+            'line1',
+            'line2',
+        ]))
+        with open(fname) as f:
+            s = f.read()
+        nt.assert_in('line1\n', s)
+        nt.assert_in('line2', s)
     
 def test_script_config():
     ip = get_ipython()
@@ -1056,7 +1192,8 @@ def test_logging_magic_quiet_from_config():
                 lm.logstart(os.path.join(td, "quiet_from_config.log"))
         finally:
             _ip.logger.logstop()
-    
+
+
 def test_logging_magic_not_quiet():
     _ip.config.LoggingMagics.quiet = False
     lm = logging.LoggingMagics(shell=_ip)
@@ -1067,14 +1204,20 @@ def test_logging_magic_not_quiet():
         finally:
             _ip.logger.logstop()
 
-## 
+
+def test_time_no_var_expand():
+    _ip.user_ns['a'] = 5
+    _ip.user_ns['b'] = []
+    _ip.magic('time b.append("{a}")')
+    assert _ip.user_ns['b'] == ['{a}']
+
+
 # this is slow, put at the end for local testing.
-## 
 def test_timeit_arguments():
     "Test valid timeit arguments, should not cause SyntaxError (GH #1269)"
     if sys.version_info < (3,7):
-        _ip.magic("timeit ('#')")
+        _ip.magic("timeit -n1 -r1 ('#')")
     else:
         # 3.7 optimize no-op statement like above out, and complain there is
         # nothing in the for loop.
-        _ip.magic("timeit a=('#')")
+        _ip.magic("timeit -n1 -r1 a=('#')")
