@@ -41,7 +41,7 @@ Give it a shot--you'll love it or you'll hate it.
 .. note::
 
   The verbose mode print all variables in the stack, which means it can
-  potentially leak sensitive information like access keys, or unencryted
+  potentially leak sensitive information like access keys, or unencrypted
   password.
 
 Installation instructions for VerboseTB::
@@ -101,10 +101,7 @@ import time
 import tokenize
 import traceback
 
-try:  # Python 2
-    generate_tokens = tokenize.generate_tokens
-except AttributeError:  # Python 3
-    generate_tokens = tokenize.tokenize
+from tokenize import generate_tokens
 
 # For purposes of monkeypatching inspect to fix a bug in it.
 from inspect import getsourcefile, getfile, getmodule, \
@@ -137,6 +134,12 @@ INDENT_SIZE = 8
 # to users of ultratb who are NOT running inside ipython.
 DEFAULT_SCHEME = 'NoColor'
 
+
+# Number of frame above which we are likely to have a recursion and will
+# **attempt** to detect it.  Made modifiable mostly to speedup test suite
+# as detecting recursion is one of our slowest test
+_FRAME_RECURSION_LIMIT = 500
+
 # ---------------------------------------------------------------------------
 # Code begins
 
@@ -431,7 +434,7 @@ def is_recursion_error(etype, value, records):
     # a recursion error.
     return (etype is recursion_error_type) \
            and "recursion" in str(value).lower() \
-           and len(records) > 500
+           and len(records) > _FRAME_RECURSION_LIMIT
 
 def find_recursion(etype, value, records):
     """Identify the repeating stack frames from a RecursionError traceback
@@ -524,6 +527,30 @@ class TBTools(colorable.Colorable):
 
     ostream = property(_get_ostream, _set_ostream)
 
+    def get_parts_of_chained_exception(self, evalue):
+        def get_chained_exception(exception_value):
+            cause = getattr(exception_value, '__cause__', None)
+            if cause:
+                return cause
+            if getattr(exception_value, '__suppress_context__', False):
+                return None
+            return getattr(exception_value, '__context__', None)
+
+        chained_evalue = get_chained_exception(evalue)
+
+        if chained_evalue:
+            return chained_evalue.__class__, chained_evalue, chained_evalue.__traceback__
+
+    def prepare_chained_exception_message(self, cause):
+        direct_cause = "\nThe above exception was the direct cause of the following exception:\n"
+        exception_during_handling = "\nDuring handling of the above exception, another exception occurred:\n"
+
+        if cause:
+            message = [[direct_cause]]
+        else:
+            message = [[exception_during_handling]]
+        return message
+
     def set_colors(self, *args, **kw):
         """Shorthand access to the color table scheme selector method."""
 
@@ -597,7 +624,13 @@ class ListTB(TBTools):
         self.ostream.write(self.text(etype, value, elist))
         self.ostream.write('\n')
 
-    def structured_traceback(self, etype, value, elist, tb_offset=None,
+    def _extract_tb(self, tb):
+        if tb:
+            return traceback.extract_tb(tb)
+        else:
+            return None
+
+    def structured_traceback(self, etype, evalue, etb=None, tb_offset=None,
                              context=5):
         """Return a color formatted string with the traceback info.
 
@@ -606,15 +639,16 @@ class ListTB(TBTools):
         etype : exception type
           Type of the exception raised.
 
-        value : object
+        evalue : object
           Data stored in the exception
 
-        elist : list
-          List of frames, see class docstring for details.
+        etb : object
+          If list: List of frames, see class docstring for details.
+          If Traceback: Traceback of the exception.
 
         tb_offset : int, optional
           Number of frames in the traceback to skip.  If not given, the
-          instance value is used (set in constructor).
+          instance evalue is used (set in constructor).
 
         context : int, optional
           Number of lines of context information to print.
@@ -623,6 +657,19 @@ class ListTB(TBTools):
         -------
         String with formatted exception.
         """
+        # This is a workaround to get chained_exc_ids in recursive calls
+        # etb should not be a tuple if structured_traceback is not recursive
+        if isinstance(etb, tuple):
+            etb, chained_exc_ids = etb
+        else:
+            chained_exc_ids = set()
+
+        if isinstance(etb, list):
+            elist = etb
+        elif etb is not None:
+            elist = self._extract_tb(etb)
+        else:
+            elist = []
         tb_offset = self.tb_offset if tb_offset is None else tb_offset
         Colors = self.Colors
         out_list = []
@@ -635,9 +682,25 @@ class ListTB(TBTools):
                             (Colors.normalEm, Colors.Normal) + '\n')
             out_list.extend(self._format_list(elist))
         # The exception info should be a single entry in the list.
-        lines = ''.join(self._format_exception_only(etype, value))
+        lines = ''.join(self._format_exception_only(etype, evalue))
         out_list.append(lines)
 
+        exception = self.get_parts_of_chained_exception(evalue)
+
+        if exception and not id(exception[1]) in chained_exc_ids:
+            chained_exception_message = self.prepare_chained_exception_message(
+                evalue.__cause__)[0]
+            etype, evalue, etb = exception
+            # Trace exception to avoid infinite 'cause' loop
+            chained_exc_ids.add(id(exception[1]))
+            chained_exceptions_tb_offset = 0
+            out_list = (
+                self.structured_traceback(
+                    etype, evalue, (etb, chained_exc_ids),
+                    chained_exceptions_tb_offset, context)
+                + chained_exception_message
+                + out_list)
+
         return out_list
 
     def _format_list(self, extracted_list):
@@ -757,7 +820,7 @@ class ListTB(TBTools):
         etype : exception type
         value : exception value
         """
-        return ListTB.structured_traceback(self, etype, value, [])
+        return ListTB.structured_traceback(self, etype, value)
 
     def show_exception_only(self, etype, evalue):
         """Only print the exception type and message, without a traceback.
@@ -1007,16 +1070,6 @@ class VerboseTB(TBTools):
                 _format_traceback_lines(lnum, index, lines, Colors, lvals,
                                          _line_format)))
 
-    def prepare_chained_exception_message(self, cause):
-        direct_cause = "\nThe above exception was the direct cause of the following exception:\n"
-        exception_during_handling = "\nDuring handling of the above exception, another exception occurred:\n"
-
-        if cause:
-            message = [[direct_cause]]
-        else:
-            message = [[exception_during_handling]]
-        return message
-
     def prepare_header(self, etype, long_version=False):
         colors = self.Colors  # just a shorthand + quicker name lookup
         colorsnormal = colors.Normal  # used a lot
@@ -1111,20 +1164,6 @@ class VerboseTB(TBTools):
             info('\nUnfortunately, your original traceback can not be constructed.\n')
             return None
 
-    def get_parts_of_chained_exception(self, evalue):
-        def get_chained_exception(exception_value):
-            cause = getattr(exception_value, '__cause__', None)
-            if cause:
-                return cause
-            if getattr(exception_value, '__suppress_context__', False):
-                return None
-            return getattr(exception_value, '__context__', None)
-
-        chained_evalue = get_chained_exception(evalue)
-
-        if chained_evalue:
-            return chained_evalue.__class__, chained_evalue, chained_evalue.__traceback__
-
     def structured_traceback(self, etype, evalue, etb, tb_offset=None,
                              number_of_lines_of_context=5):
         """Return a nice text document describing the traceback."""
@@ -1203,7 +1242,7 @@ class VerboseTB(TBTools):
                 if etb and etb.tb_next:
                     etb = etb.tb_next
                 self.pdb.botframe = etb.tb_frame
-                self.pdb.interaction(self.tb.tb_frame, self.tb)
+                self.pdb.interaction(None, etb)
 
         if hasattr(self, 'tb'):
             del self.tb
@@ -1251,7 +1290,7 @@ class FormattedTB(VerboseTB, ListTB):
                  parent=None, config=None):
 
         # NEVER change the order of this list. Put new modes at the end:
-        self.valid_modes = ['Plain', 'Context', 'Verbose']
+        self.valid_modes = ['Plain', 'Context', 'Verbose', 'Minimal']
         self.verbose_modes = self.valid_modes[1:3]
 
         VerboseTB.__init__(self, color_scheme=color_scheme, call_pdb=call_pdb,
@@ -1262,16 +1301,11 @@ class FormattedTB(VerboseTB, ListTB):
 
         # Different types of tracebacks are joined with different separators to
         # form a single string.  They are taken from this dict
-        self._join_chars = dict(Plain='', Context='\n', Verbose='\n')
+        self._join_chars = dict(Plain='', Context='\n', Verbose='\n',
+                                Minimal='')
         # set_mode also sets the tb_join_char attribute
         self.set_mode(mode)
 
-    def _extract_tb(self, tb):
-        if tb:
-            return traceback.extract_tb(tb)
-        else:
-            return None
-
     def structured_traceback(self, etype, value, tb, tb_offset=None, number_of_lines_of_context=5):
         tb_offset = self.tb_offset if tb_offset is None else tb_offset
         mode = self.mode
@@ -1280,14 +1314,15 @@ class FormattedTB(VerboseTB, ListTB):
             return VerboseTB.structured_traceback(
                 self, etype, value, tb, tb_offset, number_of_lines_of_context
             )
+        elif mode == 'Minimal':
+            return ListTB.get_exception_only(self, etype, value)
         else:
             # We must check the source cache because otherwise we can print
             # out-of-date source code.
             self.check_cache()
             # Now we can extract and format the exception
-            elist = self._extract_tb(tb)
             return ListTB.structured_traceback(
-                self, etype, value, elist, tb_offset, number_of_lines_of_context
+                self, etype, value, tb, tb_offset, number_of_lines_of_context
             )
 
     def stb2text(self, stb):
@@ -1324,6 +1359,9 @@ class FormattedTB(VerboseTB, ListTB):
     def verbose(self):
         self.set_mode(self.valid_modes[2])
 
+    def minimal(self):
+        self.set_mode(self.valid_modes[3])
+
 
 #----------------------------------------------------------------------------
 class AutoFormattedTB(FormattedTB):
@@ -1368,7 +1406,11 @@ class AutoFormattedTB(FormattedTB):
                              tb_offset=None, number_of_lines_of_context=5):
         if etype is None:
             etype, value, tb = sys.exc_info()
-        self.tb = tb
+        if isinstance(tb, tuple):
+            # tb is a tuple if this is a chained exception.
+            self.tb = tb[0]
+        else:
+            self.tb = tb
         return FormattedTB.structured_traceback(
             self, etype, value, tb, tb_offset, number_of_lines_of_context)
 
