@@ -82,11 +82,15 @@ ipython_savefig_dir:
     Sphinx source directory. The default is `html_static_path`.
 ipython_rgxin:
     The compiled regular expression to denote the start of IPython input
-    lines. The default is ``re.compile('In \[(\d+)\]:\s?(.*)\s*')``. You
+    lines. The default is ``re.compile('In \\[(\\d+)\\]:\\s?(.*)\\s*')``. You
     shouldn't need to change this.
+ipython_warning_is_error: [default to True]
+    Fail the build if something unexpected happen, for example if a block raise
+    an exception but does not have the `:okexcept:` flag. The exact behavior of
+    what is considered strict, may change between the sphinx directive version.
 ipython_rgxout:
     The compiled regular expression to denote the start of IPython output
-    lines. The default is ``re.compile('Out\[(\d+)\]:\s?(.*)\s*')``. You
+    lines. The default is ``re.compile('Out\\[(\\d+)\\]:\\s?(.*)\\s*')``. You
     shouldn't need to change this.
 ipython_promptin:
     The string to represent the IPython input prompt in the generated ReST.
@@ -184,6 +188,7 @@ To Do
 import atexit
 import errno
 import os
+import pathlib
 import re
 import sys
 import tempfile
@@ -201,11 +206,10 @@ from traitlets.config import Config
 from IPython import InteractiveShell
 from IPython.core.profiledir import ProfileDir
 
-
-use_matpltolib = False
+use_matplotlib = False
 try:
     import matplotlib
-    use_matpltolib = True
+    use_matplotlib = True
 except Exception:
     pass
 
@@ -356,7 +360,6 @@ class EmbeddedSphinxShell(object):
         self.user_ns = self.IP.user_ns
         self.user_global_ns = self.IP.user_global_ns
 
-        self.lines_waiting = []
         self.input = ''
         self.output = ''
         self.tmp_profile_dir = tmp_profile_dir
@@ -387,16 +390,16 @@ class EmbeddedSphinxShell(object):
         self.cout.seek(0)
         self.cout.truncate(0)
 
-    def process_input_line(self, line, store_history=True):
+    def process_input_line(self, line, store_history):
+        return self.process_input_lines([line], store_history=store_history)
+
+    def process_input_lines(self, lines, store_history=True):
         """process the input, capturing stdout"""
         stdout = sys.stdout
+        source_raw = '\n'.join(lines)
         try:
             sys.stdout = self.cout
-            self.lines_waiting.append(line)
-            source_raw = ''.join(self.lines_waiting)
-            if self.IP.check_complete(source_raw)[0] != 'incomplete':
-                self.lines_waiting = []
-                self.IP.run_cell(source_raw, store_history=store_history)
+            self.IP.run_cell(source_raw, store_history=store_history)
         finally:
             sys.stdout = stdout
 
@@ -413,11 +416,13 @@ class EmbeddedSphinxShell(object):
         source_dir = self.source_dir
         saveargs = decorator.split(' ')
         filename = saveargs[1]
-        # insert relative path to image file in source (as absolute path for Sphinx)
-        outfile = '/' + os.path.relpath(os.path.join(savefig_dir,filename),
-                                        source_dir)
+        # insert relative path to image file in source 
+        # as absolute path for Sphinx
+        # sphinx expects a posix path, even on Windows
+        posix_path = pathlib.Path(savefig_dir,filename).as_posix()
+        outfile = '/' + os.path.relpath(posix_path, source_dir)
 
-        imagerows = ['.. image:: %s'%outfile]
+        imagerows = ['.. image:: %s' % outfile]
 
         for kwarg in saveargs[2:]:
             arg, val = kwarg.split('=')
@@ -470,28 +475,25 @@ class EmbeddedSphinxShell(object):
 
         # Note: catch_warnings is not thread safe
         with warnings.catch_warnings(record=True) as ws:
-            for i, line in enumerate(input_lines):
-                if line.endswith(';'):
-                    is_semicolon = True
+            if input_lines[0].endswith(';'):
+                is_semicolon = True
+            #for i, line in enumerate(input_lines):
 
+            # process the first input line
+            if is_verbatim:
+                self.process_input_lines([''])
+                self.IP.execution_count += 1 # increment it anyway
+            else:
+                # only submit the line in non-verbatim mode
+                self.process_input_lines(input_lines, store_history=store_history)
+
+        if not is_suppress:
+            for i, line in enumerate(input_lines):
                 if i == 0:
-                    # process the first input line
-                    if is_verbatim:
-                        self.process_input_line('')
-                        self.IP.execution_count += 1 # increment it anyway
-                    else:
-                        # only submit the line in non-verbatim mode
-                        self.process_input_line(line, store_history=store_history)
                     formatted_line = '%s %s'%(input_prompt, line)
                 else:
-                    # process a continuation line
-                    if not is_verbatim:
-                        self.process_input_line(line, store_history=store_history)
-
                     formatted_line = '%s %s'%(continuation, line)
-
-                if not is_suppress:
-                    ret.append(formatted_line)
+                ret.append(formatted_line)
 
         if not is_suppress and len(rest.strip()) and is_verbatim:
             # The "rest" is the standard output of the input. This needs to be
@@ -557,14 +559,15 @@ class EmbeddedSphinxShell(object):
 
         # output any exceptions raised during execution to stdout
         # unless :okexcept: has been specified.
-        if not is_okexcept and "Traceback" in processed_output:
+        if not is_okexcept and (("Traceback" in processed_output) or ("SyntaxError" in processed_output)):
             s =  "\nException in %s at block ending on line %s\n" % (filename, lineno)
             s += "Specify :okexcept: as an option in the ipython:: block to suppress this message\n"
             sys.stdout.write('\n\n>>>' + ('-' * 73))
             sys.stdout.write(s)
             sys.stdout.write(processed_output)
             sys.stdout.write('<<<' + ('-' * 73) + '\n\n')
-            raise RuntimeError('Non Expected exception in `{}` line {}'.format(filename, lineno))
+            if self.warning_is_error:
+                raise RuntimeError('Non Expected exception in `{}` line {}'.format(filename, lineno))
 
         # output any warning raised during execution to stdout
         # unless :okwarning: has been specified.
@@ -579,10 +582,10 @@ class EmbeddedSphinxShell(object):
                                          w.filename, w.lineno, w.line)
                 sys.stdout.write(s)
                 sys.stdout.write('<<<' + ('-' * 73) + '\n')
-                raise RuntimeError('Non Expected warning in `{}` line {}'.format(filename, lineno))
-
-        self.cout.truncate(0)
+                if self.warning_is_error:
+                    raise RuntimeError('Non Expected warning in `{}` line {}'.format(filename, lineno))
 
+        self.clear_cout()
         return (ret, input_lines, processed_output,
                 is_doctest, decorator, image_file, image_directive)
 
@@ -734,7 +737,6 @@ class EmbeddedSphinxShell(object):
                     # will truncate tracebacks.
                     sys.stdout.write(e)
                     raise RuntimeError('An invalid block was detected.')
-
                 out_data = \
                     self.process_output(data, output_prompt, input_lines,
                                         output, is_doctest, decorator,
@@ -906,6 +908,7 @@ class IPythonDirective(Directive):
         # get regex and prompt stuff
         rgxin      = config.ipython_rgxin
         rgxout     = config.ipython_rgxout
+        warning_is_error= config.ipython_warning_is_error
         promptin   = config.ipython_promptin
         promptout  = config.ipython_promptout
         mplbackend = config.ipython_mplbackend
@@ -913,12 +916,12 @@ class IPythonDirective(Directive):
         hold_count = config.ipython_holdcount
 
         return (savefig_dir, source_dir, rgxin, rgxout,
-                promptin, promptout, mplbackend, exec_lines, hold_count)
+                promptin, promptout, mplbackend, exec_lines, hold_count, warning_is_error)
 
     def setup(self):
         # Get configuration values.
         (savefig_dir, source_dir, rgxin, rgxout, promptin, promptout,
-         mplbackend, exec_lines, hold_count) = self.get_config_options()
+         mplbackend, exec_lines, hold_count, warning_is_error) = self.get_config_options()
 
         try:
             os.makedirs(savefig_dir)
@@ -931,7 +934,7 @@ class IPythonDirective(Directive):
             # EmbeddedSphinxShell is created, its interactive shell member
             # is the same for each instance.
 
-            if mplbackend and 'matplotlib.backends' not in sys.modules and use_matpltolib:
+            if mplbackend and 'matplotlib.backends' not in sys.modules and use_matplotlib:
                 import matplotlib
                 matplotlib.use(mplbackend)
 
@@ -958,6 +961,7 @@ class IPythonDirective(Directive):
         self.shell.savefig_dir = savefig_dir
         self.shell.source_dir = source_dir
         self.shell.hold_count = hold_count
+        self.shell.warning_is_error = warning_is_error
 
         # setup bookmark for saving figures directory
         self.shell.process_input_line('bookmark ipy_savedir %s'%savefig_dir,
@@ -1008,6 +1012,15 @@ class IPythonDirective(Directive):
 
                 if figure is not None:
                     figures.append(figure)
+            else:
+                message = 'Code input with no code at {}, line {}'\
+                            .format(
+                                self.state.document.current_source,
+                                self.state.document.current_line)
+                if self.shell.warning_is_error:
+                    raise RuntimeError(message)
+                else:
+                    warnings.warn(message)
 
         for figure in figures:
             lines.append('')
@@ -1035,10 +1048,11 @@ def setup(app):
 
     app.add_directive('ipython', IPythonDirective)
     app.add_config_value('ipython_savefig_dir', 'savefig', 'env')
+    app.add_config_value('ipython_warning_is_error', True, 'env')
     app.add_config_value('ipython_rgxin',
-                         re.compile('In \[(\d+)\]:\s?(.*)\s*'), 'env')
+                         re.compile(r'In \[(\d+)\]:\s?(.*)\s*'), 'env')
     app.add_config_value('ipython_rgxout',
-                         re.compile('Out\[(\d+)\]:\s?(.*)\s*'), 'env')
+                         re.compile(r'Out\[(\d+)\]:\s?(.*)\s*'), 'env')
     app.add_config_value('ipython_promptin', 'In [%d]:', 'env')
     app.add_config_value('ipython_promptout', 'Out[%d]:', 'env')
 
@@ -1051,7 +1065,7 @@ def setup(app):
     # If the user sets this config value to `None`, then EmbeddedSphinxShell's
     # __init__ method will treat it as [].
     execlines = ['import numpy as np']
-    if use_matpltolib:
+    if use_matplotlib:
         execlines.append('import matplotlib.pyplot as plt')
     app.add_config_value('ipython_execlines', execlines, 'env')
 
